@@ -133,9 +133,22 @@
 		      (throw 'skip t)))
 		(message "Retrieving message %d (of %d) from %s..."
 			 n mailbox-count imapdrop)
-		(vm-imap-send-command process
-				      (format "FETCH %d (BODY.PEEK[])" n))
-		(vm-imap-retrieve-to-crashbox process destination statblob)
+		(condition-case data
+		    (progn
+		      (vm-imap-send-command process
+					    (format "FETCH %d (BODY.PEEK[])"
+						    n))
+		      (vm-imap-retrieve-to-crashbox process destination
+						    statblob t))
+		  (vm-imap-protocol-error
+		   (message "FETCH %d (BODY.PEEK[]) failed, trying RFC822.PEEK"
+			    n)
+		   (vm-imap-send-command process
+					 (format
+					  "FETCH %d (RFC822.PEEK)" n))
+		   (vm-imap-retrieve-to-crashbox process destination
+						 statblob nil)))
+
 		(vm-increment retrieved)
 		(and b-per-session
 		     (setq retrieved-bytes (+ retrieved-bytes message-size)))
@@ -219,7 +232,7 @@
 			  (vm-increment count))
 		      (setq response (cdr response))))
 		  (vm-store-folder-totals source (list count 0 0 0))
-		  (throw 'done (not (eq count 0))))
+		  (throw 'end-of-session (not (eq count 0))))
 		(not (equal 0 (car select)))))
 	  (setq vm-imap-retrieved-messages retrieved))
       (and process (vm-imap-end-session process)))))
@@ -473,6 +486,48 @@ on all the relevant IMAP servers and then immediately expunges."
 			  (message "IMAP password for %s incorrect" imapdrop)
 			  (sleep-for 2)
 			  (throw 'end-of-session nil))))
+		  ((equal auth "cram-md5")
+		   (let ((ipad (make-string 64 54))
+			 (opad (make-string 64 92))
+			 (command "AUTHENTICATE CRAM-MD5")
+			 (secret (concat
+				  pass (make-string (- 64 (length pass)) 0)))
+			 response p challenge answer)
+		     (vm-imap-send-command process command)
+		     (setq response (vm-imap-read-response process))
+		     (if (vm-imap-response-matches response 'VM 'NO)
+			 (error "server said NO to %s" command))
+		     (if (vm-imap-response-matches response 'VM 'BAD)
+			 (vm-imap-protocol-error "server said BAD to %s"
+						 command))
+		     (cond ((vm-imap-response-matches response '+ 'atom)
+			    (setq p (cdr (nth 1 response))
+				  challenge (buffer-substring
+					     (nth 0 p)
+					     (nth 1 p))
+				  challenge (vm-mime-base64-decode-string
+					     challenge)))
+			   (t
+			    (error "Don't understand AUTHENTICATE response")))
+		     (setq answer
+			   (concat
+			    user " "
+			    (vm-md5-string
+			     (concat
+			      (vm-xor-string secret opad)
+			      (vm-md5-raw-string 
+			       (concat
+				(vm-xor-string secret ipad) challenge)))))
+			   answer (vm-mime-base64-encode-string answer))
+		     (vm-imap-send-command process answer)
+		     (and (null (vm-imap-read-ok-response process))
+			  (progn
+			    (setq vm-imap-passwords
+				  (delete (list source-nopwd-nombox pass)
+					  vm-imap-passwords))
+			    (message "IMAP password for %s incorrect" imapdrop)
+			    (sleep-for 2)
+			    (throw 'end-of-session nil)))))
 		  ((equal auth "preauth")
 		   (if (not (eq greeting 'preauth))
 		       (progn
@@ -709,7 +764,7 @@ on all the relevant IMAP servers and then immediately expunges."
 		'skip))))
       (and work-buffer (kill-buffer work-buffer)))))
 
-(defun vm-imap-retrieve-to-crashbox (process crash statblob)
+(defun vm-imap-retrieve-to-crashbox (process crash statblob bodypeek)
   (let ((start vm-imap-read-point)
 	(need-msg t)
 	end fetch-response list p)
@@ -744,10 +799,19 @@ on all the relevant IMAP servers and then immediately expunges."
     (setq vm-imap-read-point (point-marker))
     (vm-set-imap-stat-x-got statblob nil)
     (setq list (cdr (nth 3 fetch-response)))
-    (if (not (vm-imap-response-matches list 'BODY '(vector) 'string))
-	(vm-imap-protocol-error "expected (BODY[] string) in FETCH response"))
-    (setq p (nth 2 list)
-	  start (nth 1 p))
+    (cond
+     (bodypeek
+      (if (not (vm-imap-response-matches list 'BODY '(vector) 'string))
+	  (vm-imap-protocol-error
+	   "expected (BODY[] string) in FETCH response"))
+      (setq p (nth 2 list)
+	    start (nth 1 p)))
+     (t
+      (if (not (vm-imap-response-matches list 'RFC822 'string))
+	  (vm-imap-protocol-error
+	   "expected (RFC822 string) in FETCH response"))
+      (setq p (nth 1 list)
+	    start (nth 1 p))))
     (goto-char (nth 2 p))
     (setq end (point-marker))
     (vm-imap-cleanup-region start end)
