@@ -82,7 +82,7 @@
    ))
 
 (defconst vm-menu-dispose-menu
-  (let ((title (if (vm-menu-fsfemacs-menus-p)
+  (let ((title (if (vm-menu-fsfemacs19-menus-p)
 		   (list "Dispose"
 			 "Dispose"
 			 "---"
@@ -246,7 +246,7 @@
   )
 
 (defconst vm-menu-mail-menu
-  (let ((title (if (vm-menu-fsfemacs-menus-p)
+  (let ((title (if (vm-menu-fsfemacs19-menus-p)
 		   (list "Mail Commands"
 			 "Mail Commands"
 			 "---"
@@ -261,7 +261,7 @@
 	   ["Yank Original" vm-menu-yank-original vm-reply-list]
 	   "----"
 	   (append
-	    (if (vm-menu-fsfemacs-menus-p)
+	    (if (vm-menu-fsfemacs19-menus-p)
 		(list "Send Using MIME..."
 		      "Send Using MIME..."
 		      "---"
@@ -280,7 +280,7 @@
 	      :style radio
 	      :selected (not vm-send-using-mime)]))
 	   (append
-	    (if (vm-menu-fsfemacs-menus-p)
+	    (if (vm-menu-fsfemacs19-menus-p)
 		(list "Fragment Messages Larger Than ..."
 		      "Fragment Messages Larger Than ..."
 		      "---"
@@ -328,7 +328,7 @@
 		   :style radio
 		   :selected (eq vm-mime-max-message-size 2000000)]))
 	   (append
-	    (if (vm-menu-fsfemacs-menus-p)
+	    (if (vm-menu-fsfemacs19-menus-p)
 		(list "Encode 8-bit Characters Using ..."
 		      "Encode 8-bit Characters Using ..."
 		      "---"
@@ -366,7 +366,7 @@
 	   ))))
 
 (defconst vm-menu-mime-dispose-menu
-  (let ((title (if (vm-menu-fsfemacs-menus-p)
+  (let ((title (if (vm-menu-fsfemacs19-menus-p)
 		   (list "Take Action on MIME body ..."
 			 "Take Action on MIME body ..."
 			 "---"
@@ -379,10 +379,32 @@
 	     'vm-mime-display-body-as-text) t]
 	   ["Display using External Viewer"
 	    (vm-mime-run-display-function-at-point
-	     'vm-mime-display-body-using-external-viewer) t]
-	   "---"
-	   ["Save to File" (vm-mime-run-display-function-at-point
-			    'vm-mime-send-body-to-file) t]
+	     'vm-mime-display-body-using-external-viewer) t])
+     ;; FSF Emacs does not allow a non-string menu element name.
+     (if (vm-menu-can-eval-item-name)
+	 (list [(format "Convert to %s and Display"
+			(or (nth 1 (vm-mime-can-convert
+				(car
+				 (vm-mm-layout-type
+				  (vm-mime-get-button-layout e)))))
+			    "different type"))
+		(vm-mime-run-display-function-at-point
+		 'vm-mime-convert-body-then-display)
+		(vm-mime-can-convert (car (vm-mm-layout-type
+					   (vm-mime-get-button-layout e))))]))
+     (list "---"
+	   ["Save to File" vm-mime-reader-map-save-file t]
+	   ["Save to Folder" vm-mime-reader-map-save-message
+	    (let ((layout (vm-mime-run-display-function-at-point
+			   (function
+			    (lambda (e)
+			      (vm-extent-property e 'vm-mime-layout))))))
+	      (if (null layout)
+		  nil
+		(or (vm-mime-types-match "message/rfc822"
+					 (car (vm-mm-layout-type layout)))
+		    (vm-mime-types-match "message/news"
+					 (car (vm-mm-layout-type layout))))))]
 	   ["Send to Printer" (vm-mime-run-display-function-at-point
 			       'vm-mime-send-body-to-printer) t]
 	   ["Feed to Shell Pipeline (display output)"
@@ -391,10 +413,10 @@
 	   ["Feed to Shell Pipeline (discard output)"
 	    (vm-mime-run-display-function-at-point
 	     'vm-mime-pipe-body-to-queried-command-discard-output) t]
-	   ["Delete object" 'vm-delete-mime-object t]))))
+	   ["Delete object" vm-delete-mime-object t]))))
 
 (defconst vm-menu-url-browser-menu
-  (let ((title (if (vm-menu-fsfemacs-menus-p)
+  (let ((title (if (vm-menu-fsfemacs19-menus-p)
 		   (list "Send URL to ..."
 			 "Send URL to ..."
 			 "---"
@@ -416,13 +438,17 @@
 	    (vm-mouse-send-url-at-position (point)
 					   'vm-mouse-send-url-to-mosaic)
 	    t]
+	   ["mMosaic"
+	    (vm-mouse-send-url-at-position (point)
+					   'vm-mouse-send-url-to-mmosaic)
+	    t]
 	   ["Netscape"
 	    (vm-mouse-send-url-at-position (point)
 					   'vm-mouse-send-url-to-netscape)
 	    t]))))
 
 (defconst vm-menu-mailto-url-browser-menu
-  (let ((title (if (vm-menu-fsfemacs-menus-p)
+  (let ((title (if (vm-menu-fsfemacs19-menus-p)
 		   (list "Send Mail using ..."
 			 "Send Mail using ..."
 			 "---"
@@ -433,7 +459,7 @@
      (list ["VM" (vm-mouse-send-url-at-position (point) 'ignore) t]))))
 
 (defconst vm-menu-subject-menu
-  (let ((title (if (vm-menu-fsfemacs-menus-p)
+  (let ((title (if (vm-menu-fsfemacs19-menus-p)
 		   (list "Take Action on Subject..."
 			 "Take Action on Subject..."
 			 "---"
@@ -456,7 +482,7 @@
       ))))
 
 (defconst vm-menu-author-menu
-  (let ((title (if (vm-menu-fsfemacs-menus-p)
+  (let ((title (if (vm-menu-fsfemacs19-menus-p)
 		   (list "Take Action on Author..."
 			 "Take Action on Author..."
 			 "---"
@@ -474,7 +500,7 @@
       ))))
 
 (defconst vm-menu-content-disposition-menu
-  (let ((title (if (vm-menu-fsfemacs-menus-p)
+  (let ((title (if (vm-menu-fsfemacs19-menus-p)
 		   (list "Set Content Disposition"
 			 "Set Content Disposition"
 			 "---"
@@ -503,7 +529,7 @@
 (defvar vm-menu-vm-menubar nil)
 
 (defconst vm-menu-vm-menu
-  (let ((title (if (vm-menu-fsfemacs-menus-p)
+  (let ((title (if (vm-menu-fsfemacs19-menus-p)
 		   (list "VM"
 			 "VM"
 			 "---"
@@ -696,6 +722,7 @@ set to the command name so that window configuration will be done."
 	(define-key map [rootmenu vm buffer] 'undefined)
 	(define-key map [rootmenu vm tools] 'undefined)
 	(define-key map [rootmenu vm help] 'undefined)
+	(define-key map [rootmenu vm mule] 'undefined)
 	;; 19.29 changed the tag for the Help menu.
 	(define-key map [rootmenu vm help-menu] 'undefined)
 	;; now build VM's menu tree.
