@@ -51,7 +51,7 @@
 	(pop-retrieved-messages vm-pop-retrieved-messages)
 	auto-expunge x
 	mailbox-count mailbox-size message-size response
-	n retrieved retrieved-bytes process-buffer uidl)
+	n (retrieved 0) retrieved-bytes process-buffer uidl)
     (setq auto-expunge (cond ((setq x (assoc source vm-pop-auto-expunge-alist))
 			      (cdr x))
 			     ((setq x (assoc (vm-popdrop-sans-password source)
@@ -81,7 +81,7 @@
 		(throw 'done nil))
 	    ;; loop through the maildrop retrieving and deleting
 	    ;; messages as we go.
-	    (setq n 1 retrieved 0 retrieved-bytes 0)
+	    (setq n 1 retrieved-bytes 0)
 	    (setq statblob (vm-pop-start-status-timer))
 	    (vm-set-pop-stat-x-box statblob popdrop)
 	    (vm-set-pop-stat-x-maxmsg statblob mailbox-count)
@@ -113,9 +113,17 @@
 				(throw 'skip t))))
 		      (vm-cant-uidl
 		       ;; something failed, so UIDL must not be working.
-		       ;; note that fact and carry on.
-		       (setq can-uidl nil
-			     msgid nil))))
+		       (if (and (not auto-expunge)
+				(or (not vm-pop-ok-to-ask)
+				    (not (vm-pop-ask-about-no-uidl popdrop))))
+			   (progn
+			     (message "Skipping mailbox %s (no UIDL support)"
+				      popdrop)
+			     (throw 'done (not (equal retrieved 0))))
+			 ;; user doesn't care, so go ahead and
+			 ;; expunge from the server
+			 (setq can-uidl nil
+			       msgid nil)))))
 		(vm-pop-send-command process (format "LIST %d" n))
 		(setq message-size (vm-pop-read-list-response process))
 		(vm-set-pop-stat-x-need statblob message-size)
@@ -170,6 +178,8 @@
 	      (vm-increment n))
 	     (not (equal retrieved 0)) ))
       (setq vm-pop-retrieved-messages pop-retrieved-messages)
+      (if (and (eq vm-flush-interval t) (not (equal retrieved 0)))
+	  (vm-stuff-pop-retrieved))
       (and statblob (vm-pop-stop-status-timer statblob))
       (if process
 	  (vm-pop-end-session process)))))
@@ -183,6 +193,7 @@
 			 (find-file-name-handler source)))))
 	(retrieved vm-pop-retrieved-messages)
 	(popdrop (vm-popdrop-sans-password source))
+	(count 0)
 	x response)
     (unwind-protect
 	(save-excursion
@@ -201,21 +212,24 @@
 	      (if (null (car response))
 		  ;; (nil . nil) is returned if there are no
 		  ;; messages in the mailbox.
-		  (throw 'done nil)
+		  (progn
+		    (vm-store-folder-totals source '(0 0 0 0))
+		    (throw 'done nil))
 		(while response
 		  (if (not (and (setq x (assoc (cdr (car response)) retrieved))
 				(equal (nth 1 x) popdrop)
 				(eq (nth 2 x) 'uidl)))
-		      (throw 'done t))
+		      (vm-increment count))
 		  (setq response (cdr response))))
-	      ;; all messages in the mailbox have already been retrieved
-	      (throw 'done nil))
+	      (vm-store-folder-totals source (list count 0 0 0))
+	      (throw 'done (not (eq count 0))))
 	    (vm-pop-send-command process "STAT")
 	    (setq response (vm-pop-read-stat-response process))
 	    (if (null response)
 		nil
+	      (vm-store-folder-totals source (list (car response) 0 0 0))
 	      (not (equal 0 (car response))))))
-      (and process (vm-pop-end-session process)))))
+      (and process (vm-pop-end-session process nil vm-pop-ok-to-ask)))))
 
 (defun vm-expunge-pop-messages ()
   "Deletes all messages from POP mailbox that have already been retrieved
@@ -230,7 +244,7 @@ relevant POP servers to remove the messages."
 	(source nil)
 	(trouble nil)
 	(delete-count 0)
-	(vm-block-new-mail t)
+	(vm-global-block-new-mail t)
 	(vm-pop-ok-to-ask t)
 	popdrop uidl-alist data mp match)
     (unwind-protect
@@ -329,8 +343,8 @@ relevant POP servers to remove the messages."
   (let ((process-to-shutdown nil)
 	process
 	(popdrop (vm-safe-popdrop-string source))
-	(coding-system-for-read 'binary)
-	(coding-system-for-write 'binary)
+	(coding-system-for-read (vm-binary-coding-system))
+	(coding-system-for-write (vm-binary-coding-system))
 	greeting timestamp
 	host port auth user pass source-list process-buffer source-nopwd)
     (unwind-protect
@@ -381,7 +395,7 @@ relevant POP servers to remove the messages."
 					   vm-pop-passwords)))
 	  ;; get the trace buffer
 	  (setq process-buffer
-		(generate-new-buffer (format "trace of POP session to %s"
+		(vm-make-work-buffer (format "trace of POP session to %s"
 					     host)))
 	  (save-excursion
 	    (set-buffer process-buffer)
@@ -389,8 +403,8 @@ relevant POP servers to remove the messages."
 	    ;; clear the trace buffer of old output
 	    (erase-buffer)
 	    ;; Tell MULE not to mess with the text.
-	    (if (or vm-xemacs-mule-p vm-fsfemacs-mule-p)
-		(set-buffer-file-coding-system 'binary t))
+	    (if (fboundp 'set-buffer-file-coding-system)
+		(set-buffer-file-coding-system (vm-binary-coding-system) t))
 	    (insert "starting POP session " (current-time-string) "\n")
 	    (insert (format "connecting to %s:%s\n" host port))
 	    ;; open the connection to the server
@@ -432,27 +446,53 @@ relevant POP servers to remove the messages."
 		       (progn
 			 (goto-char (point-max))
    (insert-before-markers "<<< ooops, no timestamp found in greeting! >>>\n")
+			 (message "Server of %s does not support APOP" popdrop)
+			 (sleep-for 2)
 			 (throw 'done nil)))
 		   (vm-pop-send-command
 		    process
 		    (format "APOP %s %s"
 			    user
 			    (vm-pop-md5 (concat timestamp pass))))
-		   (and (null (vm-pop-read-response process))
-			(throw 'done nil)))
+		   (if (null (vm-pop-read-response process))
+		       (progn
+			 (setq vm-pop-passwords
+			       (delete (list source-nopwd pass)
+				       vm-pop-passwords))
+			 (message "POP password for %s incorrect" popdrop)
+			 (sleep-for 2)
+			 (throw 'done nil))))
 		  (t (error "Don't know how to authenticate using %s" auth)))
 	    (setq process-to-shutdown nil)
 	    process ))
       (if process-to-shutdown
-	  (vm-pop-end-session process-to-shutdown)))))
+	  (vm-pop-end-session process-to-shutdown t)))))
 
-(defun vm-pop-end-session (process)
+(defun vm-pop-end-session (process &optional keep-buffer verbose)
   (save-excursion
     (set-buffer (process-buffer process))
     (vm-pop-send-command process "QUIT")
-    ;; we don't care about the response
-    ;;(vm-pop-read-response process)
-    (kill-buffer (process-buffer process))
+    ;; Previously we did not read the QUIT response because of
+    ;; TCP shutdown problems (under Windows?) that made it
+    ;; better if we just closed the connection.  Microsoft
+    ;; Exchange apparently fails to expunge messages if we shut
+    ;; down the connection without reading the QUIT response.
+    ;; So we provide an option and let the user decide what
+    ;; works best for them.
+    (if vm-pop-read-quit-response
+	(progn
+	  (and verbose
+	       (message "Waiting for response to POP QUIT command..."))
+	  (vm-pop-read-response process)
+	  (and verbose
+	       (message "Waiting for response to POP QUIT command... done"))))
+    (if (not keep-buffer)
+	(kill-buffer (process-buffer process))
+      (save-excursion
+       (set-buffer (process-buffer process))
+       (rename-buffer (concat "saved " (buffer-name)) t)
+       (vm-keep-some-buffers (current-buffer) 'vm-kept-pop-buffers
+			     vm-pop-keep-failed-trace-buffers)))
     (if (fboundp 'add-async-timeout)
 	(add-async-timeout 2 'delete-process process)
       (run-at-time 2 nil 'delete-process process))))
@@ -558,12 +598,15 @@ relevant POP servers to remove the messages."
 (defun vm-pop-read-stat-response (process)
   (let ((response (vm-pop-read-response process t))
 	list)
-    (setq list (vm-parse response "\\([^ ]+\\) *"))
-    (list (string-to-int (nth 1 list)) (string-to-int (nth 2 list)))))
+    (if (null response)
+	nil
+      (setq list (vm-parse response "\\([^ ]+\\) *"))
+      (list (string-to-int (nth 1 list)) (string-to-int (nth 2 list))))))
 
 (defun vm-pop-read-list-response (process)
   (let ((response (vm-pop-read-response process t)))
-    (string-to-int (nth 2 (vm-parse response "\\([^ ]+\\) *")))))
+    (and response
+	 (string-to-int (nth 2 (vm-parse response "\\([^ ]+\\) *"))))))
 
 (defun vm-pop-read-uidl-long-response (process)
   (let ((start vm-pop-read-point)
@@ -641,6 +684,26 @@ relevant POP servers to remove the messages."
 		'skip))))
       (and work-buffer (kill-buffer work-buffer)))))
 
+(defun vm-pop-ask-about-no-uidl (popdrop)
+  (let ((work-buffer nil)
+	(pop-buffer (current-buffer))
+	start end)
+    (unwind-protect
+	(save-excursion
+	  (save-window-excursion
+	    (setq work-buffer (generate-new-buffer
+			       (format "*trouble with %s*" popdrop)))
+	    (set-buffer work-buffer)
+	    (insert
+"You have asked VM to leave messages on the server for the POP mailbox "
+popdrop
+".  VM cannot do so because the server does not seem to support the POP UIDL command.\n\nYou can either continue to retrieve messages from this mailbox with VM deleting the messages from the server, or you can skip this mailbox, leaving messages on the server and not retrieving any messages.")
+	    (fill-individual-paragraphs (point-min) (point-max))
+	    (vm-display-buffer work-buffer)
+	    (setq minibuffer-scroll-window (selected-window))
+	    (yes-or-no-p "Continue retrieving anyway? ")))
+      (and work-buffer (kill-buffer work-buffer)))))
+
 (defun vm-pop-retrieve-to-crashbox (process crash statblob)
   (let ((start vm-pop-read-point) end)
     (goto-char start)
@@ -726,7 +789,7 @@ relevant POP servers to remove the messages."
   (let ((buffer nil))
     (unwind-protect
 	(save-excursion
-	  (setq buffer (generate-new-buffer "*vm-work*"))
+	  (setq buffer (vm-make-work-buffer))
 	  (set-buffer buffer)
 	  ;; call-process-region calls write-region.
 	  ;; don't let it do CR -> LF translation.
