@@ -790,7 +790,7 @@ mandatory."
 ;; The date format recognized here is the one specified in RFC 822.
 ;; Some slop is allowed e.g. dashes between the monthday, month and year
 ;; because such malformed headers have been observed.
-"\\(\\([a-z][a-z][a-z]\\),\\)?[ \t\n]*\\([0-9][0-9]?\\)[ \t\n---]*\\([a-z][a-z][a-z]\\)[ \t\n---]*\\([0-9]*[0-9][0-9]\\)[ \t\n]*\\([0-9:]+\\)[ \t\n]*\\([a-z][a-z]?[a-z]?\\|[---+]?[0-9][0-9][0-9][0-9]\\)"
+"\\(\\([a-z][a-z][a-z]\\),\\)?[ \t\n]*\\([0-9][0-9]?\\)[ \t\n---]*\\([a-z][a-z][a-z]\\)[ \t\n---]*\\([0-9]*[0-9][0-9]\\)[ \t\n]*\\([0-9:]+\\)[ \t\n]*\\([a-z][a-z]?[a-z]?\\|\\(-\\|\\+\\)[01][0-9][0-9][0-9]\\)"
        date)
       (if (match-beginning 2)
 	  (vm-su-do-weekday m (substring date (match-beginning 2)
@@ -800,13 +800,17 @@ mandatory."
       (vm-su-do-month m (substring date (match-beginning 4) (match-end 4)))
       (vm-set-year-of m (substring date (match-beginning 5) (match-end 5)))
       (if (= 2 (length (vm-year-of m)))
-	  (vm-set-year-of m (concat "19" (vm-year-of m))))
+	  (save-match-data
+	    (cond ((string-match "^[0-6]" (vm-year-of m))
+		   (vm-set-year-of m (concat "20" (vm-year-of m))))
+		  (t
+		   (vm-set-year-of m (concat "19" (vm-year-of m)))))))
       (vm-set-hour-of m (substring date (match-beginning 6) (match-end 6)))
       (vm-set-zone-of m (substring date (match-beginning 7) (match-end 7))))
      ((string-match
 ;; UNIX ctime(3) format, with slop allowed in the whitespace, and we allow for
 ;; the possibility of a timezone at the end.
-"\\([a-z][a-z][a-z]\\)[ \t\n]*\\([a-z][a-z][a-z]\\)[ \t\n]*\\([0-9][0-9]?\\)[ \t\n]*\\([0-9:]+\\)[ \t\n]*\\([0-9][0-9][0-9][0-9]\\)[ \t\n]*\\([a-z][a-z]?[a-z]?\\|[---+][0-9][0-9][0-9][0-9]\\)?"
+"\\([a-z][a-z][a-z]\\)[ \t\n]*\\([a-z][a-z][a-z]\\)[ \t\n]*\\([0-9][0-9]?\\)[ \t\n]*\\([0-9:]+\\)[ \t\n]*\\([0-9][0-9][0-9][0-9]\\)[ \t\n]*\\([a-z][a-z]?[a-z]?\\|\\(-\\|\\+\\)[01][0-9][0-9][0-9]\\)?"
        date)
       (vm-su-do-weekday m (substring date (match-beginning 1)
 				     (match-end 1)))
@@ -983,6 +987,7 @@ mandatory."
   (let ((mail-use-rfc822 t) i names addresses to cc all list full-name)
     (setq to (or (vm-get-header-contents m "To:" ", ")
 		 (vm-get-header-contents m "Apparently-To:" ", ")
+		 (vm-get-header-contents m "Newsgroups:" ", ")
 		 ;; desperation....
 		 (user-login-name))
 	  cc (vm-get-header-contents m "Cc:" ", ")
@@ -995,7 +1000,7 @@ mandatory."
       (setq full-name (or (nth 0 (funcall vm-chop-full-name-function
 					  (car list)))
 			  (car list)))
-      ;; If double quoted are around the full name, fish the name out.
+      ;; If double quotes are around the full name, fish the name out.
       (if (string-match "\\`\"\\([^\"]+\\)\"\\'" full-name)
 	  (setq full-name
 		(substring full-name (match-beginning 1) (match-end 1))))
